const std = @import("std");
const websocket = @import("websocket");

const Chatter = struct {
	client: websocket.Client,
	incoming: std.ArrayList(std.json.Value),
	outgoing: std.ArrayList(std.json.Value),
	allocator: std.mem.Allocator,

	pub fn create(allocator: std.mem.Allocator) !*Chatter {
		var self = try allocator.create(Chatter);

		self.allocator = allocator;
		self.incoming = .init(allocator);
		self.outgoing = .init(allocator);
		
		self.client = try websocket.Client.init(allocator, .{
			.host = "codeguessing.gay",
			.port = 443,
			.tls = true, // library broken?
		});
		try self.client.handshake("/73/ws", .{
			.headers = "Host: codeguessing.gay",
		});

		// spooky
		const thread = try self.client.readLoopInNewThread(self);
		thread.detach();

		return self;
	}

	// Used by readLoopInNewThread
	pub fn serverMessage(self: *Chatter, data: []u8) !void {
		std.debug.print("serverMessage: {s}\n", .{data});
		try self.incoming.append((try std.json.parseFromSlice(std.json.Value, self.allocator, data, .{})).value);
		
	}

	// Used by readLoopInNewThread
	pub fn close(self: *Chatter) void {
		self.client.close(.{}) catch unreachable;
		self.client.deinit();
		//for(self.incoming.items) |msg| msg.deinit();
		//for(self.outgoing.items) |msg| msg.deinit();
		self.incoming.deinit();
		self.outgoing.deinit();
	}

	pub fn sendMessage(self: Chatter, msg: std.json.Value) !void {
		try self.outgoing.append(msg);
	}

	pub fn recvMessage(self: *Chatter) ?std.json.Value {
		if(self.incoming.items.len == 0) return null;
		return self.incoming.orderedRemove(0);
	}
};

pub fn main() !void {
	var gpa = std.heap.GeneralPurposeAllocator(.{}){};
	const allocator = gpa.allocator();

	const chatter = try Chatter.create(allocator);
	defer allocator.destroy(chatter);
	while(true){
		const omsg = chatter.recvMessage();
		if (omsg) |msg| {
			_ = msg;
			std.debug.print("{s}", .{"a"});
		}
	}
}

